/*******************************************************************************
	Filename:       SaBLExAPI_IncomingMsg_Central
	Revised:        9/2/2015 11:38:32 AM
	Revision:       1.0

	Description:    This file contains functions used to process API messages being received by the SaBLE-x Module.

	Copyright 2015 LS Research, LLC (LSR). All rights reserved.

	IMPORTANT: Your use of this Software is limited to those specific rights
	granted under the terms of a software license agreement between the user
	who downloaded the software, his/her employer (which must be your employer)
	and LSR (the "License").  You may not use this
	Software unless you agree to abide by the terms of the License. The License
	limits your use, and you acknowledge, that the Software may not be modified,
	copied or distributed unless embedded on a Texas Instruments microcontroller
	or used solely and exclusively in conjunction with a LSR radio
	frequency transceiver, which is integrated into your product.  Other than for
	the foregoing purpose, you may not use, reproduce, copy, prepare derivative
	works of, modify, distribute, perform, display or sell this Software and/or
	its documentation for any purpose.

	YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
	PROVIDED “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
	INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
	NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
	TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
	NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
	LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
	INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
	OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
	OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
	(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

	Should you have any questions regarding your right to use this Software,
	contact LSR at www.lsr.com.
*******************************************************************************/


/*********************************************************************
* INCLUDES
*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <json.h>
#include <SaBLExAPITypes.h>
#include <SaBLExAPI_Shared.h>
#include <SaBLExAPI_IncomingMsg_Central.h>
#include <SaBLExAPI_OutgoingMsg_Central.h>
#include <SaBLExAPI_OutgoingMsg_Common.h>


/*********************************************************************
* GLOBAL VARIABLES
*/


void (*const pProcessIncomingMsgCentral[10])(JsonNode* pMsg) = {
	ProcessInvalidMessageCentral,
	AdvertisingDataReceivedIn,
	ScanCompleteIn,
	ScanResponseDataReceivedIn,
	ConnectToDeviceIn,
	QueryCentralLedBehaviorIn,
	QueryScanParamsIn,
	ScanIn,
	SetCentralLedBehaviorIn,
	SetScanParamsIn
};

/*********************************************************************
* EXTERNAL VARIABLES
*/


/*********************************************************************
* EXTERNAL FUNCTIONS
*/


/*********************************************************************
* LOCAL FUNCTIONS
*/


/*********************************************************************
* PUBLIC FUNCTIONS
*/

/*********************************************************************
 @fn	ProcessInvalidMessageCentral
 *
 @brief	This function will handle a invalid message.
 *
 @param	none
 *
 */
void ProcessInvalidMessageCentral (JsonNode* pMsg)
{

}


/*********************************************************************
 @fn	AdvertisingDataReceivedIn
 *
 @brief	This notification is sent to the host when advertising data is received.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void AdvertisingDataReceivedIn (JsonNode* pMsg)
{
	/* Get paramaters */
	params_AdvertisingDataReceivedIn_t AdvertisingDataReceivedParameters;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pParamater = json_find_member (pMsg, JSON_PARAMS_ID);
	JsonNode *pMacAddress = json_find_member (pParamater, "MacAddress");
	AdvertisingDataReceivedParameters.MacAddress = pMacAddress->string_;
	JsonNode *pAdvertisingData = json_find_member (pParamater, "AdvertisingData");
	AdvertisingDataReceivedParameters.AdvertisingData = pAdvertisingData->string_;
	JsonNode *pRSSI = json_find_member (pParamater, "RSSI");
	AdvertisingDataReceivedParameters.RSSI = pRSSI->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pRSSI);
	json_delete(pAdvertisingData);
	json_delete(pMacAddress);
	json_delete(pParamater);
	json_delete(pId);
}
/*********************************************************************
 @fn	ScanCompleteIn
 *
 @brief	Notification sent to the host to indicate if the scan has completed.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void ScanCompleteIn (JsonNode* pMsg)
{

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pId);
}
/*********************************************************************
 @fn	ScanResponseDataReceivedIn
 *
 @brief	This notification is sent to the host when scan response data is received.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void ScanResponseDataReceivedIn (JsonNode* pMsg)
{
	params_ScanResponseDataReceivedIn_t ScanResponseDataReceivedParameters;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pParamater = json_find_member (pMsg, JSON_PARAMS_ID);
	JsonNode *pMacAddress = json_find_member (pParamater, "MacAddress");
	ScanResponseDataReceivedParameters.MacAddress = pMacAddress->string_;
	JsonNode *pScanResponseData = json_find_member (pParamater, "ScanResponseData");
	ScanResponseDataReceivedParameters.ScanResponseData = pScanResponseData->string_;
	JsonNode *pRSSI = json_find_member (pParamater, "RSSI");
	ScanResponseDataReceivedParameters.RSSI = pRSSI->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pRSSI);
	json_delete(pScanResponseData);
	json_delete(pMacAddress);
	json_delete(pParamater);
	json_delete(pId);
}
/*********************************************************************
 @fn	ConnectToDeviceIn
 *
 @brief	Response to connect to device request.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void ConnectToDeviceIn (JsonNode* pMsg)
{
	results_ConnectToDeviceIn_t ConnectToDeviceResults;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pResult = json_find_member (pMsg, JSON_RESULT_ID);
	JsonNode *pStatus = json_find_member (pResult, "Status");
	ConnectToDeviceResults.Status = pStatus->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pStatus);
	json_delete(pResult);
	json_delete(pId);
}
/*********************************************************************
 @fn	QueryCentralLedBehaviorIn
 *
 @brief	Response to query central led behavior request.
All period values are in ms.
The LED on time is 50ms.  As an example, if the LED period is set to 250ms, the LED will be on for 50mS and off for 200ms.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void QueryCentralLedBehaviorIn (JsonNode* pMsg)
{
	results_QueryCentralLedBehaviorIn_t QueryCentralLedBehaviorResults;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pResult = json_find_member (pMsg, JSON_RESULT_ID);
	JsonNode *pHeartbeatLedPeriod = json_find_member (pResult, "HeartbeatLedPeriod");
	QueryCentralLedBehaviorResults.HeartbeatLedPeriod = pHeartbeatLedPeriod->number_;
	JsonNode *pUartLedActivity = json_find_member (pResult, "UartLedActivity");
	QueryCentralLedBehaviorResults.UartLedActivity = pUartLedActivity->bool_;
	JsonNode *pScanLedPeriod = json_find_member (pResult, "ScanLedPeriod");
	QueryCentralLedBehaviorResults.ScanLedPeriod = pScanLedPeriod->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pScanLedPeriod);
	json_delete(pUartLedActivity);
	json_delete(pHeartbeatLedPeriod);
	json_delete(pResult);
	json_delete(pId);
}
/*********************************************************************
 @fn	QueryScanParamsIn
 *
 @brief	Response to set the scan parameters request.  Please see the CC2640 TI software developers guide for more information about the parameters in this message.

 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void QueryScanParamsIn (JsonNode* pMsg)
{
	results_QueryScanParamsIn_t QueryScanParamsResults;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pResult = json_find_member (pMsg, JSON_RESULT_ID);
	JsonNode *pScanDuration = json_find_member (pResult, "ScanDuration");
	QueryScanParamsResults.ScanDuration = pScanDuration->number_;
	JsonNode *pMaxScanResults = json_find_member (pResult, "MaxScanResults");
	QueryScanParamsResults.MaxScanResults = pMaxScanResults->number_;
	JsonNode *pDiscoveryMode = json_find_member (pResult, "DiscoveryMode");
	QueryScanParamsResults.DiscoveryMode = pDiscoveryMode->number_;
	JsonNode *pScanMode = json_find_member (pResult, "ScanMode");
	QueryScanParamsResults.ScanMode = pScanMode->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pScanMode);
	json_delete(pDiscoveryMode);
	json_delete(pMaxScanResults);
	json_delete(pScanDuration);
	json_delete(pResult);
	json_delete(pId);
}
/*********************************************************************
 @fn	ScanIn
 *
 @brief	Response to the Scan Request.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void ScanIn (JsonNode* pMsg)
{
	results_ScanIn_t ScanResults;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pResult = json_find_member (pMsg, JSON_RESULT_ID);
	JsonNode *pStatus = json_find_member (pResult, "Status");
	ScanResults.Status = pStatus->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pStatus);
	json_delete(pResult);
	json_delete(pId);
}
/*********************************************************************
 @fn	SetCentralLedBehaviorIn
 *
 @brief	Response to set central led behavior request.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void SetCentralLedBehaviorIn (JsonNode* pMsg)
{
	results_SetCentralLedBehaviorIn_t SetCentralLedBehaviorResults;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pResult = json_find_member (pMsg, JSON_RESULT_ID);
	JsonNode *pStatus = json_find_member (pResult, "Status");
	SetCentralLedBehaviorResults.Status = pStatus->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pStatus);
	json_delete(pResult);
	json_delete(pId);
}
/*********************************************************************
 @fn	SetScanParamsIn
 *
 @brief	Response to set scan params request.
 *
 @param	pMsg - Pointer to Incoming Message
 *
 */
void SetScanParamsIn (JsonNode* pMsg)
{
	results_SetScanParamsIn_t SetScanParamsResults;

	/* Get the Id of the message */
	JsonNode *pId = json_find_member (pMsg, JSON_ID);
	uint8_t u8MsgId = pId->number_;

	JsonNode *pResult = json_find_member (pMsg, JSON_RESULT_ID);
	JsonNode *pStatus = json_find_member (pResult, "Status");
	SetScanParamsResults.Status = pStatus->number_;
	/* # App Code Start # */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pStatus);
	json_delete(pResult);
	json_delete(pId);
}

