/*******************************************************************************
	Filename:       SaBLExAPI_IncomingMsg_Central
	Revised:        9/2/2015 11:38:32 AM
	Revision:       1.0

	Description:    This file contains the SaBLE-x API message definitions and prototypes

	Copyright 2015 LS Research, LLC (LSR). All rights reserved.

	IMPORTANT: Your use of this Software is limited to those specific rights
	granted under the terms of a software license agreement between the user
	who downloaded the software, his/her employer (which must be your employer)
	and LSR (the "License").  You may not use this
	Software unless you agree to abide by the terms of the License. The License
	limits your use, and you acknowledge, that the Software may not be modified,
	copied or distributed unless embedded on a Texas Instruments microcontroller
	or used solely and exclusively in conjunction with a LSR radio
	frequency transceiver, which is integrated into your product.  Other than for
	the foregoing purpose, you may not use, reproduce, copy, prepare derivative
	works of, modify, distribute, perform, display or sell this Software and/or
	its documentation for any purpose.

	YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
	PROVIDED “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
	INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
	NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
	TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
	NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
	LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
	INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
	OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
	OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
	(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

	Should you have any questions regarding your right to use this Software,
	contact LSR at www.lsr.com.
*******************************************************************************/

#ifndef SABLEXAPI_INCOMINGMSG_CENTRAL_H
#define SABLEXAPI_INCOMINGMSG_CENTRAL_H

#ifdef __cplusplus
extern "C" {
#endif

/*********************************************************************
* INCLUDES
*/
#include <SaBLExAPITypes.h>
#include <json.h>


/*********************************************************************
* EXTERNAL VARIABLES
*/
extern void (*const pProcessIncomingMsgCentral[10])(JsonNode* pMsg);


/*********************************************************************
* CONSTANTS
*/


/*********************************************************************
* TYPEDEFS
*/

typedef struct params_AdvertisingDataReceivedIn
{
	char*	MacAddress;
	char*	AdvertisingData;
	int8_t	RSSI;
} params_AdvertisingDataReceivedIn_t;

typedef struct params_ScanResponseDataReceivedIn
{
	char*	MacAddress;
	char*	ScanResponseData;
	int8_t	RSSI;
} params_ScanResponseDataReceivedIn_t;

typedef struct results_ConnectToDeviceIn
{
	uint8_t	Status;
} results_ConnectToDeviceIn_t;

typedef struct results_QueryCentralLedBehaviorIn
{
	uint16_t	HeartbeatLedPeriod;
	bool	UartLedActivity;
	uint16_t	ScanLedPeriod;
} results_QueryCentralLedBehaviorIn_t;

typedef struct results_QueryScanParamsIn
{
	uint16_t	ScanDuration;
	uint8_t	MaxScanResults;
	uint8_t	DiscoveryMode;
	uint8_t	ScanMode;
} results_QueryScanParamsIn_t;

typedef struct results_ScanIn
{
	uint8_t	Status;
} results_ScanIn_t;

typedef struct results_SetCentralLedBehaviorIn
{
	uint8_t	Status;
} results_SetCentralLedBehaviorIn_t;

typedef struct results_SetScanParamsIn
{
	uint8_t	Status;
} results_SetScanParamsIn_t;


/*********************************************************************
* MACROS
*/


/*********************************************************************
* API FUNCTIONS
*/

/*********************************************************************
 @fn	AdvertisingDataReceivedIn
 *
 @brief	This notification is sent to the host when advertising data is received.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void AdvertisingDataReceivedIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	ScanCompleteIn
 *
 @brief	Notification sent to the host to indicate if the scan has completed.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void ScanCompleteIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	ScanResponseDataReceivedIn
 *
 @brief	This notification is sent to the host when scan response data is received.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void ScanResponseDataReceivedIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	ConnectToDeviceIn
 *
 @brief	Response to connect to device request.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void ConnectToDeviceIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	QueryCentralLedBehaviorIn
 *
 @brief	Response to query central led behavior request.
All period values are in ms.
The LED on time is 50ms.  As an example, if the LED period is set to 250ms, the LED will be on for 50mS and off for 200ms.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void QueryCentralLedBehaviorIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	QueryScanParamsIn
 *
 @brief	Response to set the scan parameters request.  Please see the CC2640 TI software developers guide for more information about the parameters in this message.

 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void QueryScanParamsIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	ScanIn
 *
 @brief	Response to the Scan Request.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void ScanIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	SetCentralLedBehaviorIn
 *
 @brief	Response to set central led behavior request.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void SetCentralLedBehaviorIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	SetScanParamsIn
 *
 @brief	Response to set scan params request.
 *
 @param	JsonNode* pMsg - Pointer to Incoming Message
 *
 */
void SetScanParamsIn ( JsonNode* pMsg);
/*********************************************************************
 @fn	ProcessInvalidMessageCentral
 *
 @brief	This function will handle a invalid message.
 *
 @param	none
 *
 */
void ProcessInvalidMessageCentral (JsonNode* pMsg);


#ifdef __cplusplus
}
#endif

#endif /*SABLEXAPI_INCOMINGMSG_CENTRAL_H*/
