/*******************************************************************************
	Filename:       SaBLExAPI_OutgoingMsg_Central
	Revised:        9/2/2015 11:38:32 AM
	Revision:       1.0

	Description:    This file contains functions used to generate API messages being transmitted by the SaBLE-x Module.

	Copyright 2015 LS Research, LLC (LSR). All rights reserved.

	IMPORTANT: Your use of this Software is limited to those specific rights
	granted under the terms of a software license agreement between the user
	who downloaded the software, his/her employer (which must be your employer)
	and LSR (the "License").  You may not use this
	Software unless you agree to abide by the terms of the License. The License
	limits your use, and you acknowledge, that the Software may not be modified,
	copied or distributed unless embedded on a Texas Instruments microcontroller
	or used solely and exclusively in conjunction with a LSR radio
	frequency transceiver, which is integrated into your product.  Other than for
	the foregoing purpose, you may not use, reproduce, copy, prepare derivative
	works of, modify, distribute, perform, display or sell this Software and/or
	its documentation for any purpose.

	YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
	PROVIDED “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
	INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
	NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
	TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
	NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
	LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
	INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
	OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
	OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
	(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

	Should you have any questions regarding your right to use this Software,
	contact LSR at www.lsr.com.
*******************************************************************************/


/*********************************************************************
* INCLUDES
*/
#include <FreeRtos.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <json.h>
#include <SaBLExAPI_Shared.h>
#include <SaBLExAPI_OutgoingMsg_Central.h>


/*********************************************************************
* GLOBAL VARIABLES
*/


/*********************************************************************
* EXTERNAL VARIABLES
*/


/*********************************************************************
* EXTERNAL FUNCTIONS
*/


/*********************************************************************
* LOCAL FUNCTIONS
*/


/*********************************************************************
* PUBLIC FUNCTIONS
*/
/*********************************************************************
 @fn	QueryCentralLedBehaviorOut
 *
 @brief	Request to query the LED behavior for the central.
 *
 @param	u8MsgId - Id of the Received Message
 *
 */
void QueryCentralLedBehaviorOut (uint8_t u8MsgId)
{
	JsonNode *pMsg = json_mkobject();

	/* Create message */
	json_append_member (pMsg, JSON_HEADER_STRING, json_mkstring(JSON_HEADER_VALUE));
	json_append_member (pMsg, JSON_METHOD_ID, json_mkstring("QueryCentralLedBehavior"));
	json_append_member (pMsg, JSON_ID, json_mknumber(u8MsgId));
	char* pTemp = json_stringify (pMsg, NULL);

	/* # App Code Start # */
	/* Send pTemp out UART */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pMsg);
	vPortFree(pTemp);
}


/*********************************************************************
 @fn	SetCentralLedBehaviorOut
 *
 @brief	Request to set the LED behavior for the central.
All period values are in ms.
The LED on time is 50ms.  As an example, if the LED period is set to 250ms, the LED will be on for 50mS and off for 200ms.

 *
 @param	u8MsgId - Id of the Received Message
 @param	params_SetCentralLedBehaviorOut - Point to structure that represent the message.
 *
 */
void SetCentralLedBehaviorOut (uint8_t u8MsgId, params_SetCentralLedBehaviorOut_t* params_SetCentralLedBehaviorOut)
{
	JsonNode *pMsg = json_mkobject();

	JsonNode *pParam = json_mkobject();

	/* Fill in message parameters */
	json_append_member(pParam, "HeartbeatLedPeriod", json_mknumber(params_SetCentralLedBehaviorOut->HeartbeatLedPeriod));
	json_append_member(pParam, "UartLedActivity", json_mkbool(params_SetCentralLedBehaviorOut->UartLedActivity));
	json_append_member(pParam, "ScanLedPeriod", json_mknumber(params_SetCentralLedBehaviorOut->ScanLedPeriod));
	json_append_member(pParam, "SaveToNv", json_mkbool(params_SetCentralLedBehaviorOut->SaveToNv));
	/* Fill in message parameters end*/

	/* Create message */
	json_append_member (pMsg, JSON_HEADER_STRING, json_mkstring(JSON_HEADER_VALUE));
	json_append_member (pMsg, JSON_METHOD_ID, json_mkstring("SetCentralLedBehavior"));
	json_append_member (pMsg, JSON_PARAMS_ID, pParam);
	json_append_member (pMsg, JSON_ID, json_mknumber(u8MsgId));
	char* pTemp = json_stringify (pMsg, NULL);

	/* # App Code Start # */
	/* Send pTemp out UART */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pParam);
	json_delete(pMsg);
	vPortFree(pTemp);
}


/*********************************************************************
 @fn	ConnectToDeviceOut
 *
 @brief	Request to connect to the specified MAC address.
 *
 @param	u8MsgId - Id of the Received Message
 @param	params_ConnectToDeviceOut - Point to structure that represent the message.
 *
 */
void ConnectToDeviceOut (uint8_t u8MsgId, params_ConnectToDeviceOut_t* params_ConnectToDeviceOut)
{
	JsonNode *pMsg = json_mkobject();

	JsonNode *pParam = json_mkobject();

	/* Fill in message parameters */
	json_append_member(pParam, "MacAddress", json_mkstring(params_ConnectToDeviceOut->MacAddress));
	/* Fill in message parameters end*/

	/* Create message */
	json_append_member (pMsg, JSON_HEADER_STRING, json_mkstring(JSON_HEADER_VALUE));
	json_append_member (pMsg, JSON_METHOD_ID, json_mkstring("ConnectToDevice"));
	json_append_member (pMsg, JSON_PARAMS_ID, pParam);
	json_append_member (pMsg, JSON_ID, json_mknumber(u8MsgId));
	char* pTemp = json_stringify (pMsg, NULL);

	/* # App Code Start # */
	/* Send pTemp out UART */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pParam);
	json_delete(pMsg);
	vPortFree(pTemp);
}


/*********************************************************************
 @fn	QueryScanParamsOut
 *
 @brief	Request to query the scan params.
 *
 @param	u8MsgId - Id of the Received Message
 *
 */
void QueryScanParamsOut (uint8_t u8MsgId)
{
	JsonNode *pMsg = json_mkobject();

	/* Create message */
	json_append_member (pMsg, JSON_HEADER_STRING, json_mkstring(JSON_HEADER_VALUE));
	json_append_member (pMsg, JSON_METHOD_ID, json_mkstring("QueryScanParams"));
	json_append_member (pMsg, JSON_ID, json_mknumber(u8MsgId));
	char* pTemp = json_stringify (pMsg, NULL);

	/* # App Code Start # */
	/* Send pTemp out UART */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pMsg);
	vPortFree(pTemp);
}


/*********************************************************************
 @fn	ScanOut
 *
 @brief	Enable or disable the device scan based on the parameters in the SetScanParams message.

On boot this setting is disabled by default.
 *
 @param	u8MsgId - Id of the Received Message
 @param	params_ScanOut - Point to structure that represent the message.
 *
 */
void ScanOut (uint8_t u8MsgId, params_ScanOut_t* params_ScanOut)
{
	JsonNode *pMsg = json_mkobject();

	JsonNode *pParam = json_mkobject();

	/* Fill in message parameters */
	json_append_member(pParam, "Enable", json_mkbool(params_ScanOut->Enable));
	/* Fill in message parameters end*/

	/* Create message */
	json_append_member (pMsg, JSON_HEADER_STRING, json_mkstring(JSON_HEADER_VALUE));
	json_append_member (pMsg, JSON_METHOD_ID, json_mkstring("Scan"));
	json_append_member (pMsg, JSON_PARAMS_ID, pParam);
	json_append_member (pMsg, JSON_ID, json_mknumber(u8MsgId));
	char* pTemp = json_stringify (pMsg, NULL);

	/* # App Code Start # */
	/* Send pTemp out UART */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pParam);
	json_delete(pMsg);
	vPortFree(pTemp);
}


/*********************************************************************
 @fn	SetScanParamsOut
 *
 @brief	Request to set the scan parameters.  Please see the CC2640 TI software developers guide for more information about the parameters in this message.

 *
 @param	u8MsgId - Id of the Received Message
 @param	params_SetScanParamsOut - Point to structure that represent the message.
 *
 */
void SetScanParamsOut (uint8_t u8MsgId, params_SetScanParamsOut_t* params_SetScanParamsOut)
{
	JsonNode *pMsg = json_mkobject();

	JsonNode *pParam = json_mkobject();

	/* Fill in message parameters */
	json_append_member(pParam, "ScanDuration", json_mknumber(params_SetScanParamsOut->ScanDuration));
	json_append_member(pParam, "MaxScanResults", json_mknumber(params_SetScanParamsOut->MaxScanResults));
	json_append_member(pParam, "DiscoveryMode", json_mknumber(params_SetScanParamsOut->DiscoveryMode));
	json_append_member(pParam, "ScanMode", json_mknumber(params_SetScanParamsOut->ScanMode));
	json_append_member(pParam, "SaveToNv", json_mkbool(params_SetScanParamsOut->SaveToNv));
	/* Fill in message parameters end*/

	/* Create message */
	json_append_member (pMsg, JSON_HEADER_STRING, json_mkstring(JSON_HEADER_VALUE));
	json_append_member (pMsg, JSON_METHOD_ID, json_mkstring("SetScanParams"));
	json_append_member (pMsg, JSON_PARAMS_ID, pParam);
	json_append_member (pMsg, JSON_ID, json_mknumber(u8MsgId));
	char* pTemp = json_stringify (pMsg, NULL);

	/* # App Code Start # */
	/* Send pTemp out UART */

	/* # App Code End # */

	/* Clean Up Memory */
	json_delete(pParam);
	json_delete(pMsg);
	vPortFree(pTemp);
}



