/*******************************************************************************
	Filename:       SaBLExAPI_OutgoingMsg_Central
	Revised:        9/2/2015 11:38:32 AM
	Revision:       1.0

	Description:    This file contains the SaBLE-x API outgoing message definitions and prototypes

	Copyright 2015 LS Research, LLC (LSR). All rights reserved.

	IMPORTANT: Your use of this Software is limited to those specific rights
	granted under the terms of a software license agreement between the user
	who downloaded the software, his/her employer (which must be your employer)
	and LSR (the "License").  You may not use this
	Software unless you agree to abide by the terms of the License. The License
	limits your use, and you acknowledge, that the Software may not be modified,
	copied or distributed unless embedded on a Texas Instruments microcontroller
	or used solely and exclusively in conjunction with a LSR radio
	frequency transceiver, which is integrated into your product.  Other than for
	the foregoing purpose, you may not use, reproduce, copy, prepare derivative
	works of, modify, distribute, perform, display or sell this Software and/or
	its documentation for any purpose.

	YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
	PROVIDED “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
	INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
	NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
	TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
	NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
	LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
	INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
	OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
	OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
	(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

	Should you have any questions regarding your right to use this Software,
	contact LSR at www.lsr.com.
*******************************************************************************/

#ifndef SABLEXAPI_OUTGOINGMSG_CENTRAL_H
#define SABLEXAPI_OUTGOINGMSG_CENTRAL_H

#ifdef __cplusplus
extern "C" {
#endif

/*********************************************************************
* INCLUDES
*/
#include <SaBLExAPITypes.h>


/*********************************************************************
* EXTERNAL VARIABLES
*/


/*********************************************************************
* CONSTANTS
*/


/*********************************************************************
* TYPEDEFS
*/

typedef struct params_SetCentralLedBehaviorOut
{
	uint16_t	HeartbeatLedPeriod;
	bool	UartLedActivity;
	uint16_t	ScanLedPeriod;
	bool	SaveToNv;
} params_SetCentralLedBehaviorOut_t;

typedef struct params_ConnectToDeviceOut
{
	char*	MacAddress;
} params_ConnectToDeviceOut_t;

typedef struct params_ScanOut
{
	bool	Enable;
} params_ScanOut_t;

typedef struct params_SetScanParamsOut
{
	uint16_t	ScanDuration;
	uint8_t	MaxScanResults;
	uint8_t	DiscoveryMode;
	uint8_t	ScanMode;
	bool	SaveToNv;
} params_SetScanParamsOut_t;


/*********************************************************************
* MACROS
*/


/*********************************************************************
* API FUNCTIONS
*/

/*********************************************************************
 @fn	QueryCentralLedBehaviorOut
 *
 @brief	Request to query the LED behavior for the central.
 *
 @param	uint8_t u8MsgId - Message Id
 *
 */
void QueryCentralLedBehaviorOut ( uint8_t u8MsgId);
/*********************************************************************
 @fn	SetCentralLedBehaviorOut
 *
 @brief	Request to set the LED behavior for the central.
All period values are in ms.
The LED on time is 50ms.  As an example, if the LED period is set to 250ms, the LED will be on for 50mS and off for 200ms.

 *
 @param	uint8_t u8MsgId - Message Id
 @param	params_SetCentralLedBehaviorOut - Point to structure that represent the message.
 *
 */
void SetCentralLedBehaviorOut ( uint8_t u8MsgId, params_SetCentralLedBehaviorOut_t* params_SetCentralLedBehaviorOut);
/*********************************************************************
 @fn	ConnectToDeviceOut
 *
 @brief	Request to connect to the specified MAC address.
 *
 @param	uint8_t u8MsgId - Message Id
 @param	params_ConnectToDeviceOut - Point to structure that represent the message.
 *
 */
void ConnectToDeviceOut ( uint8_t u8MsgId, params_ConnectToDeviceOut_t* params_ConnectToDeviceOut);
/*********************************************************************
 @fn	QueryScanParamsOut
 *
 @brief	Request to query the scan params.
 *
 @param	uint8_t u8MsgId - Message Id
 *
 */
void QueryScanParamsOut ( uint8_t u8MsgId);
/*********************************************************************
 @fn	ScanOut
 *
 @brief	Enable or disable the device scan based on the parameters in the SetScanParams message.

On boot this setting is disabled by default.
 *
 @param	uint8_t u8MsgId - Message Id
 @param	params_ScanOut - Point to structure that represent the message.
 *
 */
void ScanOut ( uint8_t u8MsgId, params_ScanOut_t* params_ScanOut);
/*********************************************************************
 @fn	SetScanParamsOut
 *
 @brief	Request to set the scan parameters.  Please see the CC2640 TI software developers guide for more information about the parameters in this message.

 *
 @param	uint8_t u8MsgId - Message Id
 @param	params_SetScanParamsOut - Point to structure that represent the message.
 *
 */
void SetScanParamsOut ( uint8_t u8MsgId, params_SetScanParamsOut_t* params_SetScanParamsOut);

#ifdef __cplusplus
}
#endif

#endif /*SABLEXAPI_OUTGOINGMSG_CENTRAL_H*/
