#include "FreeRTOS.h"
#include <string.h>
#include "circularBuffer.h"

static uint16_t cb_count(circular_buffer_t *cb);

void cb_init(circular_buffer_t *cb, uint16_t capacity)
{
    cb->buffer = pvPortMalloc(capacity );
    if(cb->buffer == NULL)
    {
        for(;;);
    }
    cb->buffer_end = cb->buffer + capacity;
    cb->capacity = capacity;
    cb->head = cb->buffer;
    cb->tail = cb->buffer;
}

void cb_free(circular_buffer_t *cb)
{
    vPortFree(cb->buffer);
}

void cb_push_back(circular_buffer_t *cb, const char *item)
{
    if(cb_count(cb) >= cb->capacity)
    {
        for(;;);
    }
    memcpy((void*)cb->head, item, 1);
    cb->head++;
    if(cb->head >= cb->buffer_end)
    {
        cb->head = cb->buffer;
    }
}

void cb_pop_front(circular_buffer_t *cb, char *item)
{
    if(cb_count(cb) <= 0)
    {
        return;
    }
    memcpy(item, (void*)cb->tail, 1);
    cb->tail++;
    if(cb->tail >= cb->buffer_end)
    {
        cb->tail = cb->buffer;
    }
}

void cb_pop_front_no_read(circular_buffer_t *cb)
{
    if(cb_count(cb) <= 0)
    {
        return;
    }
    cb->tail++;
    if(cb->tail >= cb->buffer_end)
    {
        cb->tail = cb->buffer;
    }
}

static uint16_t cb_count(circular_buffer_t *cb)
{
    uint16_t u16Count = 0;
    if(cb->head > cb->tail)
    {
        u16Count = (cb->head - cb->tail);
    }
    else if(cb->tail > cb->head)
    {
        u16Count = (cb->capacity - (cb->tail - cb->head));
    } 
    
    return u16Count;
}