#include "FreeRTOS.h"
#include "task.h"

#include "json.h"
#include "SaBLExAPI_Shared.h"
#include "SaBLExAPI_IncomingMsg_Common.h"
#include <stdbool.h>
#include <string.h>
#include "timers.h"
#include "circularBuffer.h"
#include "mspUart.h"
#include "circularBuffer.h"
#include "uartProcessor.h"

#define uartProcessorSTACK_SIZE				configMINIMAL_STACK_SIZE * 3

#define RX_BUFFER_CAPACITY  512
#define TX_BUFFER_CAPACITY  512
#define CRC_OVERHEAD        12
static circular_buffer_t rxBuffer;

const uint8_t hextable[256] = {
  //[0 ... 255] = -1, // bit aligned access into this table is considerably
  ['0'] = 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, // faster for most modern processors,
  ['A'] = 10, 11, 12, 13, 14, 15,       // for the space conscious, reduce to
  ['a'] = 10, 11, 12, 13, 14, 15        // signed char.
};


static volatile char* pcCurrentByte;

static const uint16_t crcTable[256]  =
{
    0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
    0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
    0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6,
    0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de,
    0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485,
    0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d,
    0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4,
    0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc,
    0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823,
    0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b,
    0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12,
    0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a,
    0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41,
    0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49,
    0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70,
    0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78,
    0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f,
    0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067,
    0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e,
    0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256,
    0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d,
    0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
    0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c,
    0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634,
    0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab,
    0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3,
    0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a,
    0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92,
    0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9,
    0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1,
    0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
    0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
};

TaskHandle_t xUartTaskHandle;
TimerHandle_t xUartTimer;
/* The receive task as described at the top of the file. */
static portTASK_FUNCTION_PROTO( vComRxTask, pvParameters );

static uint16_t UartProcessor_crcByte(uint16_t u16fcs, uint8_t u8c);
static uint16_t UartProcessor_calculateCrc(char* pcMessage);

static void UartProcessor_IncrementCurrentByte(void);
static bool UartProcessor_BufferContainsStringAtCurrentByte(char* pcString);
static uint16_t UartProcessor_DistanceBetweenCurrentByteAndBufferHead(void);

static void UartProcessor_lookForStartByte(void);
static void UartProcessor_validateHeader(void);
static void UartProcessor_validateTrailer(void);
static void UartProcessor_parseMsg(void);

static void (*UartProcessorRxStateMachine)(void) = UartProcessor_lookForStartByte;

/*-----------------------------------------------------------*/

void vTimerCallback( TimerHandle_t pxTimer )
 {
    rxBuffer.tail = rxBuffer.head;

    UartProcessorRxStateMachine = UartProcessor_lookForStartByte;
 }

void UartProcessor_StartTask( UBaseType_t uxPriority, uint32_t ulBaudRate)
{
    cb_init( &rxBuffer, RX_BUFFER_CAPACITY );
	MspUart_Init( ulBaudRate, TX_BUFFER_CAPACITY );
	xTaskCreate( vComRxTask, "COMRx", uartProcessorSTACK_SIZE, NULL, uxPriority, xUartTaskHandle );
    uint16_t u16TimerId = 1;
    xUartTimer = xTimerCreate
                   (  /* Just a text name, not used by the RTOS kernel. */
                     "UartTimer",
                     /* The timer period in ticks, must be greater than 0. */
                     /* Tick rate is 1000Hz (1ms)...500ms timeout. */
                     500,
                     /* The timers will auto-reload themselves when they
                     expire. */
                     pdFALSE,
                     /* Assign each timer a unique id equal to its array
                     index. */
                     ( void * ) u16TimerId,
                     /* Each timer calls the same callback when it expires. */
                     vTimerCallback
                   );    
}

void UartProcessor_WriteTxMessage(char* pcMessage)
{
    //calculate crc; pcBuffer is already null terminated so
    //we can use this function
    uint16_t u16crc = UartProcessor_calculateCrc(pcMessage);
    char res[4];
    res[0] = TO_HEX(((u16crc & 0xF000) >> 12));   
    res[1] = TO_HEX(((u16crc & 0x0F00) >> 8));
    res[2] = TO_HEX(((u16crc & 0x00F0) >> 4));
    res[3] = TO_HEX((u16crc & 0x000F));

    uint16_t u16MsgSize = strlen(pcMessage) + CRC_OVERHEAD;
    
    char* pcMsg = pvPortMalloc(u16MsgSize);
    //copy pcBuffer into pcMsg prior to bracketizing
    memcpy(pcMsg+1, pcMessage, strlen(pcMessage));
    
    //bracketize and add crc
    pcMsg[0] = '{';
    pcMsg[u16MsgSize-1] = '}';
    pcMsg[u16MsgSize-2] = '}';
    pcMsg[u16MsgSize-3] = res[3];
    pcMsg[u16MsgSize-4] = res[2];
    pcMsg[u16MsgSize-5] = res[1];
    pcMsg[u16MsgSize-6] = res[0];
    pcMsg[u16MsgSize-7] = ':';
    pcMsg[u16MsgSize-8] = 'C';
    pcMsg[u16MsgSize-9] = 'R';
    pcMsg[u16MsgSize-10] = 'C';
    pcMsg[u16MsgSize-11] = '{';
    
    MspUart_Write( pcMsg, u16MsgSize );
    
    vPortFree(pcMsg);
}

/*-----------------------------------------------------------*/

static portTASK_FUNCTION( vComRxTask, pvParameters )
{
uint32_t ulNotificationValue;
	/* Just to stop compiler warnings. */
	( void ) pvParameters;
    MspUart_RegisterTaskHandleAndBuffers(&rxBuffer);
	for( ;; )
	{
        /* Wait to be notified that the transmission is complete.  Note the first
        parameter is pdTRUE, which has the effect of clearing the task's notification
        value back to 0, making the notification value act like a binary (rather than
        a counting) semaphore.  */
        ulNotificationValue = ulTaskNotifyTake( pdFALSE,
                                                portMAX_DELAY );

        if( ulNotificationValue == 1 )
        {
            if(rxBuffer.tail != rxBuffer.head)
            {
                UartProcessorRxStateMachine();
            }
        }
	}
} /*lint !e715 !e818 pvParameters is required for a task function even if it is not referenced. */
/*-----------------------------------------------------------*/
static uint16_t UartProcessor_crcByte(uint16_t u16fcs, uint8_t u8c)
{
    uint8_t index = (uint8_t)((u16fcs >> 8 ^ u8c) & 0xff);
    
    u16fcs = (uint16_t)(crcTable[index] ^ (u16fcs << 8));
    return u16fcs;
}

static uint16_t UartProcessor_calculateCrc(char* pcMessage)
{
    int len = strlen(pcMessage);
    uint16_t fcs = 0xffff;
    
    for(int i =0; i<len; i++)
    {
        fcs = UartProcessor_crcByte(fcs, pcMessage[i]);
    }
    
    return fcs;
}

static void UartProcessor_IncrementCurrentByte(void)
{
    pcCurrentByte++;
    
    if(pcCurrentByte >= rxBuffer.buffer_end)
    {
        pcCurrentByte = rxBuffer.buffer;
    }  
}

/*********************************************************************
* @fn      CircularBuffer_ContainsStringAtCurrentByte
*
* @brief   Determines if buffer contains a string at the current byte.
*          Spaces used to omit checking a character
*
* @param   pcString - The string to check for.
*
* @return  True if found.
*/
static bool UartProcessor_BufferContainsStringAtCurrentByte(char* pcString)
{

    for(uint16_t i =0; i< strlen(pcString); i++)
    {
        if(pcString[i] != ' ')
        {
            if(pcString[i] != *pcCurrentByte)
            {
                if(*pcCurrentByte != pcString[0])
                {
                    UartProcessor_IncrementCurrentByte();
                }
                return false;
            }
            else
            {
               UartProcessor_IncrementCurrentByte(); 
            }
        }
        else
        {
            UartProcessor_IncrementCurrentByte(); 
        }
    }        

    return true;
}

/*********************************************************************
* @fn      CircularBuffer_DistanceBetweenCurrentByteAndHead
*
* @brief   Determines how far away the working pointer is from the head
*
* @param   None.
*
* @return  Distance in bytes.
*/
static uint16_t UartProcessor_DistanceBetweenCurrentByteAndBufferHead(void)
{
    if(pcCurrentByte > rxBuffer.head)
    {
        return (RX_BUFFER_CAPACITY - (pcCurrentByte - rxBuffer.head));

    }
    else if(pcCurrentByte < rxBuffer.head)
    {
        return (rxBuffer.head - pcCurrentByte);
    }  
    else
    {
        return 0;
    }
}


static void UartProcessor_lookForStartByte(void)
{
    while(*rxBuffer.tail != '{' && (rxBuffer.tail != rxBuffer.head))
    {
        cb_pop_front_no_read(&rxBuffer);
    }    
    
    if(*rxBuffer.tail == '{')
    {
        pcCurrentByte = rxBuffer.tail;
        xTimerStart( xUartTimer, 0 );
        UartProcessorRxStateMachine = UartProcessor_validateHeader;
    } 
    
    xTaskNotifyGive(xUartTaskHandle);
}

/*********************************************************************
* @fn      UartProcessor_validateHeader
*
* @brief   Validates the rx header if enough bytes are present.
*
* @param   None.
*
* @return  None.
*/
static void UartProcessor_validateHeader(void)
{
    if(UartProcessor_DistanceBetweenCurrentByteAndBufferHead() >= 17)
    {
        if(UartProcessor_BufferContainsStringAtCurrentByte("{{\"jsonrpc\":\"2.0\""))
        {
            UartProcessorRxStateMachine = UartProcessor_validateTrailer;
        }
        else
        {     
            xTimerStop( xUartTimer, 0 );
            rxBuffer.tail = pcCurrentByte;

            UartProcessorRxStateMachine = UartProcessor_lookForStartByte;
        }
    }
    xTaskNotifyGive(xUartTaskHandle);
}

/*********************************************************************
* @fn      UartProcessor_validateTrailer
*
* @brief   Validates the rx trailer if enough bytes are present.
*
* @param   None.
*
* @return  None.
*/
static void UartProcessor_validateTrailer(void)
{
    if(UartProcessor_DistanceBetweenCurrentByteAndBufferHead() >= 12 )
    {    
        //spaces denote don't cares for crc values, checked later
        if(UartProcessor_BufferContainsStringAtCurrentByte("}{CRC:    }}"))
        {
            UartProcessor_parseMsg();
            UartProcessorRxStateMachine = UartProcessor_lookForStartByte;
        }
    }
    xTaskNotifyGive(xUartTaskHandle);    
}

/*********************************************************************
* @fn      UartProcessor_parseMsg
*
* @brief   Parses the received message.
*
* @param   None.
*
* @return  None.
*/
static void UartProcessor_parseMsg(void)
{
    xTimerStop( xUartTimer, 0 );
    uint16_t u16MsgSize;
    static uint16_t u16crc;
    if(pcCurrentByte > rxBuffer.tail)
    {
        u16MsgSize = (pcCurrentByte - rxBuffer.tail);
    }
    else
    {
        u16MsgSize = (RX_BUFFER_CAPACITY - (rxBuffer.tail - pcCurrentByte));
    }
    
    char* pcBuffer = pvPortMalloc(u16MsgSize);
    
    for(uint16_t i = 0; i<u16MsgSize; i++)
    {
        cb_pop_front(&rxBuffer, &pcBuffer[i]);
    }
    pcCurrentByte = rxBuffer.tail;
    //Extract CRC
    u16crc = hextable[pcBuffer[u16MsgSize-3]];
    u16crc |= (hextable[pcBuffer[u16MsgSize-4]]) << 4;
    u16crc |= (hextable[pcBuffer[u16MsgSize-5]]) << 8;
    u16crc |= (hextable[pcBuffer[u16MsgSize-6]]) << 12;
    
    //12 chracters of overhead for crc; +1 for null terminator
    uint16_t u16JsonMsgSize = u16MsgSize -11;
    char* pcJsonMessage = pvPortMalloc(u16JsonMsgSize);
    
    //don't include last char reserved for null terminator
    for(uint16_t i =0; i<(u16JsonMsgSize-1); i++)
    {
        pcJsonMessage[i] = pcBuffer[i+1];
    }
    
    //add null terminator
    pcJsonMessage[u16JsonMsgSize-1] = 0;
    
    //pcBuffer not needed anymore
    vPortFree(pcBuffer);
    //function requires null termination
    static uint16_t u16MsgCrc = 0;
    u16MsgCrc = UartProcessor_calculateCrc(pcJsonMessage);
    
    if(u16crc == u16MsgCrc)
    {
        char pcErrorMsg[256];
        JsonNode *message = json_decode(pcJsonMessage);
        
        if(json_check(message, pcErrorMsg))
        {

            if(strstr(pcJsonMessage, "method")!=NULL)
            {  
                JsonNode *method = json_find_member(message, "method");
                MessageNamespace_t msgNamespace;
                uint8_t u8MsgIdx = GetIncomingMsgIndex (method->string_, &msgNamespace);
                
                if(u8MsgIdx > 0)
                {              
                    if(msgNamespace == Common)
                    {
                        (*pProcessIncomingMsgCommon[u8MsgIdx])(message);
                    }
                }
                else
                {
                    //handle error
                }
                json_delete(method);
            }
            else if(strstr(pcJsonMessage, "result")!=NULL)
            {
                JsonNode *id = json_find_member(message, "id");
                if(GetExpectedMsgFunction(id->number_) != NULL)
                {
                    GetExpectedMsgFunction(id->number_)(message);
                }
                else
                {
                    json_delete(id); 
                }
                
            }
        }
        else
        {
            //handle error
        }
        json_delete(message);
    }
    else
    {
        //handle errror
    }
    
    vPortFree(pcJsonMessage); 
}